{#
  Copyright (C) 2019-2020 CERN.
  Copyright (C) 2019-2020 Northwestern University.

  Invenio App RDM is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}
{%- extends "invenio_theme/frontpage.html" %}

{%- block page_header %}
{%- include "invenio_app_rdm/header_frontpage.html" %}
{%- endblock page_header %}

{%- block page_body %}
{%- block first_section%}
<section class="get-started-section section-content-white-bg">
  <span class="section-title frontpage-title">{{ _("You've successfully installed InvenioRDM! What is ahead?") }}</span>
  <div class="ui container">
    <div class="ui padded relaxed centered grid">
      <div class="invenio-rdm-project-goals">
        <div class="rdm-goal">
          <div class="flex-center-vertically">
            <div class="numbering">1</div>
          </div>
          <h3 class="section-title">{{ _("Configure") }}</h3>
          <div class="command-line">
            {% trans configuration_link="https://inveniordm.docs.cern.ch/install/configuration/" %}
              Configure the datamodel, resource types, permissions and much more to make it suit your needs! For more
              information on how to do it visit the <a href="{{ configuration_link }}" target="_blank">configuration
              section in the docs</a>.
            {% endtrans %}
          </div>
        </div>
        <div class="rdm-goal">
          <div class="flex-center-vertically">
            <div class="numbering">2</div>
          </div>
          <h3 class="section-title">{{ _("Customize") }}</h3>
            <div class="command-line">
              {% trans configuration_link="https://inveniordm.docs.cern.ch/customize/styling/" %}
                Customize the layout and adapt the styling to match your institution's. See how to do so <a
                href="{{ configuration_link }}" target="_blank">here</a>.
              {% endtrans %}
          </div>
        </div>
        <div class="rdm-goal">
          <div class="flex-center-vertically">
            <div class="numbering">3</div>
          </div>
          <h3 class="section-title">{{ _("Deploy it!") }}</h3>
          <div class="command-line">
            {% trans deployment_link="https://inveniordm.docs.cern.ch/deployment/" %}
              Use the Helm charts to deploy your InvenioRDM instance. You can use the Helm chart to
              <a href="{{ deployment_link }}" target="_blank">deploy in OpenShift</a>.
            {% endtrans %}
          </div>
        </div>
      </div>
    </div>
  </div>
  <div class="row inline-row">
    <div class="inline-row-pane">
      <div class="goto-button">
        <a href="https://inveniordm.docs.cern.ch" target="_blank"><i class="file alternate outline icon"></i>{{ _("Docs") }}</a>
      </div>
      <div class="goto-button">
        <a href="https://github.com/inveniosoftware" target="_blank"><i class="github icon"></i>{{ _("Code") }}</a>
      </div>
    </div>
  </div>
</section>
{%- endblock first_section%}
{%- endblock page_body%}
